/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.geoviewer.jxmap.map.cache.spi;

import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import nl.cloudfarming.client.geoviewer.jxmap.map.cache.TileCacheInfo;
import nl.cloudfarming.client.geoviewer.jxmap.map.cache.TileService;
import nl.cloudfarming.client.util.FileUtil;
import org.openide.filesystems.FileObject;

/**
 *
 * @author Frantisek Post
 */
public abstract class AbstractTileService implements TileService {

    private static final Logger LOG = Logger.getLogger(AbstractTileService.class.getName());

    public BufferedImage readImage(TileCacheInfo tileCacheInfo) {
        BufferedImage image = null;
        FileObject tileFile = getTileFile(tileCacheInfo, false);
        if (tileFile != null) {
            try (InputStream stream = tileFile.getInputStream()) {
                image = ImageIO.read(stream);
                readCallback(tileFile);
            } catch (IOException ex) {
                LOG.log(Level.FINE, "Error reading tile image from file ", ex);
            }
        }
        return image;
    }

    public void writeImage(TileCacheInfo tileCacheInfo, BufferedImage image) {
        FileObject tileFile = getTileFile(tileCacheInfo, true);
        if (tileFile != null) {
            try (OutputStream stream = tileFile.getOutputStream()) {
                ImageIO.write(image, "png", stream);
                writeCallback();
            } catch (IOException ex) {
                LOG.log(Level.FINE, "Problem writing tile image to file", ex);
            }
        } else {
            LOG.fine("Failed to write image for tile");
        }
    }

    protected FileObject getTileFile(TileCacheInfo tileCacheInfo, boolean createMissing) {
        FileObject zoomFolder = getZoomFolder(tileCacheInfo, createMissing);
        FileObject tileFile = null;
        if (zoomFolder != null) {
            tileFile = zoomFolder.getFileObject(tileCacheInfo.getPath());
            if (tileFile == null && createMissing) {
                try {
                    tileFile = zoomFolder.createData(tileCacheInfo.getPath());
                } catch (IOException ex) {
                    LOG.log(Level.FINE, "Error creating tile file " + tileCacheInfo.getPath(), ex);
                }
            }
        }
        return tileFile;
    }

    protected FileObject getZoomFolder(TileCacheInfo tileCacheInfo, boolean createMissing) {
        FileObject zoomFolder = null;
        String zoomFolderName = String.format("%d", tileCacheInfo.getZoom());
        FileObject rootFolder = CacheManager.getInstance().getCacheFolder();
        if (rootFolder != null) {
            zoomFolder = rootFolder.getFileObject(zoomFolderName);
            if (zoomFolder == null && createMissing) {
                try {
                    zoomFolder = rootFolder.createFolder(zoomFolderName);
                } catch (IOException ex) {
                    LOG.log(Level.FINE, "Error creating folder " + zoomFolderName, ex);
                }
            }
        }
        return zoomFolder;
    }

    protected void readCallback(FileObject tileFile) {
        try {
            FileUtil.touch(tileFile); 
        } catch (IOException ex) {
            LOG.log(Level.FINE, "Error while touching file", ex);
        }
    }

    protected void writeCallback() {
        //noop
    }
}
