/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.geoviewer.jxmap.map.cache.spi;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.geoviewer.jxmap.map.cache.options.OfflineCachePreferences;
import org.openide.filesystems.FileObject;

/**
 * 
 * @author Frantisek Post
 */
public class CacheCleaner {

    private static final Logger LOG = Logger.getLogger(CacheCleaner.class.getName());
    private static CacheCleaner instance = new CacheCleaner();
    
    public static final long DEFAULT_SIZE = 20 * 1024 * 1024; // 20 MB initial size
    public static final long UNLIMITED_SIZE = -1;
    
    private boolean running = false; //to avoid calling twice or more - tiles are loaded from different threads
    private Comparator<FileObject> comparator;
    protected long maximumCacheSize = OfflineCachePreferences.CACHE_LIMIT.getValue();
    
    protected CacheCleaner() {
        comparator = new Comparator<FileObject>() {
            
            @Override
            public int compare(FileObject file1, FileObject file2) {
                long lastModified1 = file1.lastModified().getTime();
                long lastModified2 = file2.lastModified().getTime();
                if (lastModified1 > lastModified2) {
                    return -1;
                } else if (lastModified1 < lastModified2) {
                    return 1;
                } else {
                    return 0;
                }
            }
            
        };
    }

    private List<FileObject> findCacheFiles(FileObject root) {
        List<FileObject> files = new ArrayList<FileObject>();
        findCacheFiles(root, files);
        return files;
    }
    
    private void findCacheFiles(FileObject file, List<FileObject> files) {
        if (file != null) {
            if (file.isFolder()) {
                FileObject[] children = file.getChildren(); 
                for (FileObject child : children) {
                    findCacheFiles(child, files);
                }
            } else {
                files.add(file);
            }
        }
    }
    
    private List<FileObject> getFilesToDelete(List<FileObject> files) {
        long sizeSummary = 0;
        List<FileObject> filesToDelete = new ArrayList<FileObject>();
        for (FileObject file : files) {
            sizeSummary += file.getSize();
            if (sizeSummary > maximumCacheSize) {
                filesToDelete.add(file);
            } 
        }
        return filesToDelete;
    }
    
    private void deleteFiles(List<FileObject> files) {
        List<FileObject> filesToDelete = getFilesToDelete(files);
        for (FileObject file : filesToDelete) {
            safelyDelete(file);
        }
    }
    
    
    private void safelyDelete(FileObject file) {
        try {
            file.delete();
        } catch (Throwable t) {
            LOG.log(Level.FINER, "unable to delete cache file {0}, exception {1}", new Object[] {file.getPath(), t.getMessage()});
        }
    }
    
    protected void runInternal() {
        running = true;
        try {
            List<FileObject> files = findCacheFiles(getRootDir());
            Collections.sort(files, comparator);
            deleteFiles(files);
        } finally {
            running = false;
        }
        
    }
    
    protected FileObject getRootDir() {
        return CacheManager.getInstance().getCacheFolder();
    }
    
    /// API ///
    
    public void run() {
        if (running || isUnlimited()) {
            return;
        }
        
        Thread thread = new Thread(new Runnable() {
            
            @Override
            public void run() {
                runInternal();
            }
            
        });
        thread.setName("Cache deleting thread");
        thread.start();
    }
    
    public static CacheCleaner getInstance() {
        return instance;
    }

    public void setMaximumCacheSize(long maximumCacheSize) {
        long oldSize = this.maximumCacheSize;
        this.maximumCacheSize = maximumCacheSize;
        //in case, the newly set maximum size is smaller than actual, check the files and delete necessary
        //or in case the cache was unlimited and I am setting limit
        if ((maximumCacheSize < oldSize && !isUnlimited()) || (oldSize == UNLIMITED_SIZE && !isUnlimited())) {
            run();
        }
    }

    public boolean isUnlimited() {
        return maximumCacheSize == UNLIMITED_SIZE;
    }
}
